# coding=utf-8
import datetime
import logging
import os
import os.path
import traceback

import timeout_decorator
from django_cron import CronJobBase, Schedule

from web.models import ConfigurationItem
from web.models.constants import CRON_JOB_TIMEOUT, DEFAULT_FROM_EMAIL, DAILY_SUBJECT_IMPORT_FILE, \
    DAILY_VISIT_IMPORT_FILE, SUBJECT_IMPORT_RUN_AT, VISIT_IMPORT_RUN_AT
from web.smash_email import EmailSender
from .csv_tns_subject_import_reader import TnsCsvSubjectImportReader
from .csv_tns_visit_import_reader import TnsCsvVisitImportReader
from .importer import Importer

logger = logging.getLogger(__name__)


class SubjectImporterCronJob(CronJobBase):
    item = ConfigurationItem.objects.filter(type=SUBJECT_IMPORT_RUN_AT).first()
    RUN_AT_TIMES = []
    if item is not None:
        RUN_AT_TIMES = item.value.split(';')
    schedule = Schedule(run_at_times=RUN_AT_TIMES)
    code = 'web.import_subjects_daily_job'  # a unique code

    @timeout_decorator.timeout(CRON_JOB_TIMEOUT)
    def do(self):
        email_title = "Subjects daily import"
        email_recipients = ConfigurationItem.objects.get(type=DEFAULT_FROM_EMAIL).value

        filename = ConfigurationItem.objects.get(type=DAILY_SUBJECT_IMPORT_FILE).value

        if filename is None or filename == '':
            logger.info("Importing subjects skipped. File not defined ")
            return "import file not defined"
        logger.info("Importing subjects from file: " + filename)
        if not os.path.isfile(filename):
            EmailSender().send_email(email_title,
                                     "<h3><font color='red'>File with imported data is not available in the system: " + filename + "</font></h3>",
                                     email_recipients)
            return "import file not found"
        try:
            importer = Importer(filename, TnsCsvSubjectImportReader())
            importer.execute()
            email_body = importer.get_summary()
            EmailSender().send_email(email_title,
                                     "<h3>Data was successfully imported from file: " + filename + "</h3>" + email_body,
                                     email_recipients)
            self.backup_file(filename)
            return "import is successful"

        except:
            tb = traceback.format_exc()
            EmailSender().send_email(email_title,
                                     "<h3><font color='red'>There was a problem with importing data from file: " + filename + "</font></h3><pre>" + tb + "</pre>",
                                     email_recipients)
            return "import crashed"

    def backup_file(self, filename):
        new_file = filename + "-" + datetime.datetime.now().strftime("%Y-%m-%d-%H-%M") + ".bac"
        os.rename(filename, new_file)
        return


class VisitImporterCronJob(CronJobBase):
    item = ConfigurationItem.objects.filter(type=VISIT_IMPORT_RUN_AT).first()
    RUN_AT_TIMES = []
    if item is not None:
        RUN_AT_TIMES = item.value.split(';')
    schedule = Schedule(run_at_times=RUN_AT_TIMES)
    code = 'web.import_visits_daily_job'  # a unique code

    @timeout_decorator.timeout(CRON_JOB_TIMEOUT)
    def do(self):
        email_title = "Visits daily import"
        email_recipients = ConfigurationItem.objects.get(type=DEFAULT_FROM_EMAIL).value

        filename = ConfigurationItem.objects.get(type=DAILY_VISIT_IMPORT_FILE).value

        if filename is None or filename == '':
            logger.info("Importing visits skipped. File not defined ")
            return "import file not defined"
        logger.info("Importing visits from file: " + filename)
        if not os.path.isfile(filename):
            EmailSender().send_email(email_title,
                                     "<h3><font color='red'>File with imported data is not available in the system: " + filename + "</font></h3>",
                                     email_recipients)
            return "import file not found"
        try:
            importer = TnsCsvVisitImportReader()
            importer.load_data(filename)
            email_body = importer.get_summary()
            EmailSender().send_email(email_title,
                                     "<h3>Data was successfully imported from file: " + filename + "</h3>" + email_body,
                                     email_recipients)
            self.backup_file(filename)
            return "import is successful"

        except:
            tb = traceback.format_exc()
            EmailSender().send_email(email_title,
                                     "<h3><font color='red'>There was a problem with importing data from file: " + filename + "</font></h3><pre>" + tb + "</pre>",
                                     email_recipients)
            return "import crashed"

    def backup_file(self, filename):
        new_file = filename + "-" + datetime.datetime.now().strftime("%Y-%m-%d-%H-%M") + ".bac"
        os.rename(filename, new_file)
        return
