# coding=utf-8

import logging

import pytz
from django.test import TestCase
from django.utils import timezone

from web.importer import TnsCsvVisitImportReader, MsgCounterHandler
from web.models import Appointment, Visit, StudySubject, AppointmentTypeLink, AppointmentType, ConfigurationItem
from web.models.constants import IMPORT_APPOINTMENT_TYPE
from web.tests.functions import get_resource_path, create_study_subject, create_appointment_type, create_location

logger = logging.getLogger(__name__)


class TestTnsCsvVisitReader(TestCase):
    def setUp(self):
        self.warning_counter = MsgCounterHandler()
        logging.getLogger('').addHandler(self.warning_counter)

        item = ConfigurationItem.objects.get(type=IMPORT_APPOINTMENT_TYPE)
        item.value = "SAMPLE_2"
        item.save()
        create_appointment_type(code="SAMPLE_2")

        create_study_subject(nd_number='cov-000111')
        create_study_subject(nd_number='cov-222333')
        create_study_subject(nd_number='cov-444444')

        create_location(name="Laboratoires réunis	23 Route de Diekirch	6555	Bollendorf-Pont")
        create_location(name="PickenDoheem")
        create_location(name="Ketterthill	1-3, rue de la Continentale 	4917	Bascharage")

    def tearDown(self):
        logging.getLogger('').removeHandler(self.warning_counter)

    def test_load_data(self):
        filename = get_resource_path('tns_vouchers_import.csv')
        visits = TnsCsvVisitImportReader().load_data(filename)
        self.assertEqual(3, len(visits))
        visit = Visit.objects.filter(id=visits[0].id)[0]
        self.assertEqual("cov-000111", visit.subject.nd_number)
        self.assertEqual(1, visit.visit_number)

        appointment = Appointment.objects.filter(visit=visit)[0]
        self.assertEqual("Laboratoires réunis	23 Route de Diekirch	6555	Bollendorf-Pont",
                         appointment.location.name)

        self.assertEqual(10, appointment.datetime_when.day)
        self.assertEqual(4, appointment.datetime_when.month)
        self.assertEqual(2020, appointment.datetime_when.year)

        self.assertEqual(0, self.get_warnings_count())

    def test_load_data_with_existing_visit(self):
        filename = get_resource_path('tns_vouchers_import.csv')
        visit = Visit.objects.create(subject=StudySubject.objects.filter(nd_number='cov-000111')[0],
                                     datetime_end=timezone.now(),
                                     datetime_begin=timezone.now(),
                                     visit_number=1)
        visits = TnsCsvVisitImportReader().load_data(filename)
        visit = Visit.objects.filter(id=visits[0].id)[0]
        self.assertEqual("cov-000111", visit.subject.nd_number)

        appointment = Appointment.objects.filter(visit=visit)[0]
        self.assertEqual("Laboratoires réunis	23 Route de Diekirch	6555	Bollendorf-Pont",
                         appointment.location.name)

        self.assertEqual(10, appointment.datetime_when.day)
        self.assertEqual(4, appointment.datetime_when.month)
        self.assertEqual(2020, appointment.datetime_when.year)

        self.assertEqual(0, self.get_warnings_count())

    def test_load_data_with_existing_visit_and_appointment(self):
        filename = get_resource_path('tns_vouchers_import.csv')
        visit = Visit.objects.create(subject=StudySubject.objects.filter(nd_number='cov-000111')[0],
                                     datetime_end=timezone.now(),
                                     datetime_begin=timezone.now(),
                                     visit_number=1)
        appointment = Appointment.objects.create(visit=visit, length=1, datetime_when=timezone.now(),
                                                 location=create_location())

        AppointmentTypeLink.objects.create(appointment_id=appointment.id,
                                           appointment_type=AppointmentType.objects.filter(code="SAMPLE_2")[0])

        visits = TnsCsvVisitImportReader().load_data(filename)
        visit = Visit.objects.filter(id=visits[0].id)[0]
        self.assertEqual("cov-000111", visit.subject.nd_number)

        self.assertEqual(1, Appointment.objects.filter(visit=visit).count())

        appointment = Appointment.objects.filter(visit=visit)[0]
        self.assertEqual("Laboratoires réunis	23 Route de Diekirch	6555	Bollendorf-Pont",
                         appointment.location.name)

        self.assertEqual(10, appointment.datetime_when.day)
        self.assertEqual(4, appointment.datetime_when.month)
        self.assertEqual(2020, appointment.datetime_when.year)

        self.assertEqual(0, self.get_warnings_count())

    def test_load_data_with_visit_and_no_previous_visits(self):
        filename = get_resource_path('tns_vouchers_3_import.csv')

        visits = TnsCsvVisitImportReader().load_data(filename)

        subject_visits = Visit.objects.filter(subject=StudySubject.objects.filter(nd_number='cov-000111')[0])

        self.assertEqual(3, len(subject_visits))

        visit = Visit.objects.filter(id=visits[0].id)[0]
        self.assertEqual("cov-000111", visit.subject.nd_number)

        appointment = Appointment.objects.filter(visit=visit)[0]
        self.assertEqual("Laboratoires réunis	23 Route de Diekirch	6555	Bollendorf-Pont",
                         appointment.location.name)

        self.assertEqual(10, appointment.datetime_when.day)
        self.assertEqual(4, appointment.datetime_when.month)
        self.assertEqual(2020, appointment.datetime_when.year)

        self.assertEqual(2, self.get_warnings_count())

    def test_load_data_with_no_subject(self):
        filename = get_resource_path('tns_vouchers_import.csv')
        StudySubject.objects.filter(nd_number="cov-000111").delete()
        visits = TnsCsvVisitImportReader().load_data(filename)
        self.assertEqual(3, len(visits))
        visit = Visit.objects.filter(id=visits[0].id)[0]
        self.assertEqual("cov-000111", visit.subject.nd_number)
        self.assertEqual(1, visit.visit_number)

        self.assertEqual(0, self.get_warnings_count())

    def test_load_data_with_lab_id(self):
        filename = get_resource_path('tns_vouchers_lab_id_import.csv')
        visits = TnsCsvVisitImportReader().load_data(filename)
        self.assertEqual(3, len(visits))

        visit = Visit.objects.filter(id=visits[0].id)[0]
        appointment = Appointment.objects.filter(visit=visit)[0]
        self.assertTrue("Laboratoires réunis" in appointment.location.name)

        visit = Visit.objects.filter(id=visits[1].id)[0]
        appointment = Appointment.objects.filter(visit=visit)[0]
        self.assertTrue("Ketterthill" in appointment.location.name)

        visit = Visit.objects.filter(id=visits[2].id)[0]
        appointment = Appointment.objects.filter(visit=visit)[0]
        self.assertTrue("BioneXt" in appointment.location.name)

        self.assertEqual(3, self.get_warnings_count())

    def test_dont_add_links_for_existing_appointments(self):
        filename = get_resource_path('tns_vouchers_import.csv')
        TnsCsvVisitImportReader().load_data(filename)
        links = AppointmentTypeLink.objects.all().count()

        TnsCsvVisitImportReader().load_data(filename)
        self.assertEqual(links, AppointmentTypeLink.objects.all().count())

        self.assertEqual(0, self.get_warnings_count())

    def get_warnings_count(self):
        if "WARNING" in self.warning_counter.level2count:
            return self.warning_counter.level2count["WARNING"]
        else:
            return 0
